﻿// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// Microsoft Public License (MS-PL)
// 
// This license governs use of the accompanying software. If you use the
// software, you accept this license. If you do not accept the license, do not
// use the software.
// 
// 1. Definitions
// 
//   The terms "reproduce," "reproduction," "derivative works," and
//   "distribution" have the same meaning here as under U.S. copyright law. A
//   "contribution" is the original software, or any additions or changes to
//   the software. A "contributor" is any person that distributes its
//   contribution under this license. "Licensed patents" are a contributor's
//   patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// 
//   (A) Copyright Grant- Subject to the terms of this license, including the
//       license conditions and limitations in section 3, each contributor
//       grants you a non-exclusive, worldwide, royalty-free copyright license
//       to reproduce its contribution, prepare derivative works of its
//       contribution, and distribute its contribution or any derivative works
//       that you create.
// 
//   (B) Patent Grant- Subject to the terms of this license, including the
//       license conditions and limitations in section 3, each contributor
//       grants you a non-exclusive, worldwide, royalty-free license under its
//       licensed patents to make, have made, use, sell, offer for sale,
//       import, and/or otherwise dispose of its contribution in the software
//       or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// 
//   (A) No Trademark License- This license does not grant you rights to use
//       any contributors' name, logo, or trademarks.
// 
//   (B) If you bring a patent claim against any contributor over patents that
//       you claim are infringed by the software, your patent license from such
//       contributor to the software ends automatically.
// 
//   (C) If you distribute any portion of the software, you must retain all
//       copyright, patent, trademark, and attribution notices that are present
//       in the software.
// 
//   (D) If you distribute any portion of the software in source code form, you
//       may do so only under this license by including a complete copy of this
//       license with your distribution. If you distribute any portion of the
//       software in compiled or object code form, you may only do so under a
//       license that complies with this license.
// 
//   (E) The software is licensed "as-is." You bear the risk of using it. The
//       contributors give no express warranties, guarantees or conditions. You
//       may have additional consumer rights under your local laws which this
//       license cannot change. To the extent permitted under your local laws,
//       the contributors exclude the implied warranties of merchantability,
//       fitness for a particular purpose and non-infringement.
//       

#pragma once

//-----------------------------------------------------------------------------
// V8ContextImpl
//-----------------------------------------------------------------------------

class V8ContextImpl: public V8Context
{
    PROHIBIT_COPY(V8ContextImpl)

public:

    V8ContextImpl(V8IsolateImpl* pIsolateImpl, const wchar_t* pName, bool enableDebugging, bool disableGlobalMembers, int debugPort);

    void CallWithLock(LockCallbackT* pCallback, void* pvArg);
    V8Value GetRootObject();
    void SetGlobalProperty(const wchar_t* pName, const V8Value& value, bool globalMembers);
    V8Value Execute(const wchar_t* pDocumentName, const wchar_t* pCode, bool discard);
    V8ScriptHolder* Compile(const wchar_t* pDocumentName, const wchar_t* pCode);
    bool CanExecute(V8ScriptHolder* pHolder);
    V8Value Execute(V8ScriptHolder* pHolder);
    void Interrupt();
    void GetIsolateHeapInfo(V8IsolateHeapInfo& heapInfo);
    void CollectGarbage(bool exhaustive);

    void* AddRefV8Object(void* pvObject);
    void ReleaseV8Object(void* pvObject);

    V8Value GetV8ObjectProperty(void* pvObject, const wchar_t* pName);
    void SetV8ObjectProperty(void* pvObject, const wchar_t* pName, const V8Value& value);
    bool DeleteV8ObjectProperty(void* pvObject, const wchar_t* pName);
    void GetV8ObjectPropertyNames(void* pvObject, vector<wstring>& names);

    V8Value GetV8ObjectProperty(void* pvObject, int index);
    void SetV8ObjectProperty(void* pvObject, int index, const V8Value& value);
    bool DeleteV8ObjectProperty(void* pvObject, int index);
    void GetV8ObjectPropertyIndices(void* pvObject, vector<int>& indices);

    V8Value InvokeV8Object(void* pvObject, const vector<V8Value>& args, bool asConstructor);
    V8Value InvokeV8ObjectMethod(void* pvObject, const wchar_t* pName, const vector<V8Value>& args);

    void ProcessDebugMessages();

    ~V8ContextImpl();

private:

    class Scope
    {
        PROHIBIT_COPY(Scope)
        PROHIBIT_HEAP(Scope)

    public:

        explicit Scope(V8ContextImpl* pContextImpl):
            m_pContextImpl(pContextImpl),
            m_ContextScope(m_pContextImpl->m_hContext)
        {
        }

        ~Scope()
        {
            if (!uncaught_exception() && m_pContextImpl->m_hContext->HasOutOfMemoryException())
            {
                m_pContextImpl->m_spIsolateImpl->ThrowOutOfMemoryException();
            }
        }

    private:

        V8ContextImpl* m_pContextImpl;
        Context::Scope m_ContextScope;
    };

    Local<Context> CreateContext(ExtensionConfiguration* pExtensionConfiguation = nullptr, Handle<ObjectTemplate> hGlobalTemplate = Handle<ObjectTemplate>(), Handle<Value> hGlobalObject = Handle<Value>())
    {
        return m_spIsolateImpl->CreateContext(pExtensionConfiguation, hGlobalTemplate, hGlobalObject);
    }

    Handle<Primitive> GetUndefined()
    {
        return m_spIsolateImpl->GetUndefined();
    }

    Handle<Primitive> GetNull()
    {
        return m_spIsolateImpl->GetNull();
    }

    Handle<Boolean> GetTrue()
    {
        return m_spIsolateImpl->GetTrue();
    }

    Handle<Boolean> GetFalse()
    {
        return m_spIsolateImpl->GetFalse();
    }

    Local<Integer> CreateInteger(__int32 value)
    {
        return m_spIsolateImpl->CreateInteger(value);
    }

    Local<Integer> CreateInteger(unsigned __int32 value)
    {
        return m_spIsolateImpl->CreateInteger(value);
    }

    template <typename T>
    Local<T> CreateLocal(Handle<T> hTarget)
    {
        return m_spIsolateImpl->CreateLocal(hTarget);
    }

    template <typename T>
    Local<T> CreateLocal(Persistent<T> hTarget)
    {
        return m_spIsolateImpl->CreateLocal(hTarget);
    }

    template <typename T>
    Persistent<T> CreatePersistent(Handle<T> hTarget)
    {
        return m_spIsolateImpl->CreatePersistent(hTarget);
    }

    template <typename T>
    Persistent<T> CreatePersistent(Persistent<T> hTarget)
    {
        return m_spIsolateImpl->CreatePersistent(hTarget);
    }

    template <typename T, typename TArg>
    Persistent<T> MakeWeak(Persistent<T> hTarget, TArg* pArg, void (*pCallback)(Isolate*, Persistent<T>*, TArg*))
    {
        return m_spIsolateImpl->MakeWeak(hTarget, pArg, pCallback);
    }

    template <typename T>
    void Dispose(Persistent<T> hTarget)
    {
        return m_spIsolateImpl->Dispose(hTarget);
    }

    void TerminateExecution()
    {
        return m_spIsolateImpl->TerminateExecution();
    }

    template <typename T>
    T Verify(const TryCatch& tryCatch, T result)
    {
        Verify(tryCatch);
        return result;
    }

    Handle<Value> Wrap();

    void GetV8ObjectPropertyNames(Handle<Object> hObject, vector<wstring>& names);
    void GetV8ObjectPropertyIndices(Handle<Object> hObject, vector<int>& indices);

    static void GetGlobalProperty(Local<String> hName, const PropertyCallbackInfo<Value>& info);
    static void SetGlobalProperty(Local<String> hName, Local<Value> value, const PropertyCallbackInfo<Value>& info);
    static void QueryGlobalProperty(Local<String> hName, const PropertyCallbackInfo<Integer>& info);
    static void DeleteGlobalProperty(Local<String> hName, const PropertyCallbackInfo<Boolean>& info);
    static void GetGlobalPropertyNames(const PropertyCallbackInfo<Array>& info);

    static void GetGlobalProperty(unsigned __int32 index, const PropertyCallbackInfo<Value>& info);
    static void SetGlobalProperty(unsigned __int32 index, Local<Value> hValue, const PropertyCallbackInfo<Value>& info);
    static void QueryGlobalProperty(unsigned __int32 index, const PropertyCallbackInfo<Integer>& info);
    static void DeleteGlobalProperty(unsigned __int32 index, const PropertyCallbackInfo<Boolean>& info);
    static void GetGlobalPropertyIndices(const PropertyCallbackInfo<Array>& info);

    static void GetHostObjectProperty(Local<String> hName, const PropertyCallbackInfo<Value>& info);
    static void SetHostObjectProperty(Local<String> hName, Local<Value> hValue, const PropertyCallbackInfo<Value>& info);
    static void QueryHostObjectProperty(Local<String> hName, const PropertyCallbackInfo<Integer>& info);
    static void DeleteHostObjectProperty(Local<String> hName, const PropertyCallbackInfo<Boolean>& info);
    static void GetHostObjectPropertyNames(const PropertyCallbackInfo<Array>& info);

    static void GetHostObjectProperty(unsigned __int32 index, const PropertyCallbackInfo<Value>& info);
    static void SetHostObjectProperty(unsigned __int32 index, Local<Value> hValue, const PropertyCallbackInfo<Value>& info);
    static void QueryHostObjectProperty(unsigned __int32 index, const PropertyCallbackInfo<Integer>& info);
    static void DeleteHostObjectProperty(unsigned __int32 index, const PropertyCallbackInfo<Boolean>& info);
    static void GetHostObjectPropertyIndices(const PropertyCallbackInfo<Array>& info);

    static void InvokeHostObject(const FunctionCallbackInfo<Value>& info);
    static void DisposeWeakHandle(Isolate* pIsolate, Persistent<Object>* phObject, void* pvV8ObjectCache);

    Handle<Value> ImportValue(const V8Value& value);
    V8Value ExportValue(Handle<Value> hValue);
    void ImportValues(const vector<V8Value>& values, vector<Handle<Value>>& importedValues);

    void Verify(const TryCatch& tryCatch);
    void VerifyNotOutOfMemory();
    void ThrowScriptException(const HostException& exception);

    wstring m_Name;
    SharedPtr<V8IsolateImpl> m_spIsolateImpl;
    Persistent<Context> m_hContext;
    Persistent<Object> m_hGlobal;
    vector<Persistent<Object>> m_GlobalMembersStack;
    Persistent<String> m_hHostObjectCookieName;
    Persistent<String> m_hInnerExceptionName;
    Persistent<FunctionTemplate> m_hHostObjectTemplate;
    void* m_pvV8ObjectCache;
};
